@@ -200,7 +200,7 @@ get_default_menu_item(di_slist *list)
 		}
 		if (p->installer_menu_item < last_successful_item &&
 		    di_hash_table_lookup(seen_items, &p->p.key)) {
-			//di_log(DI_LOG_LEVEL_DEBUG, "menu item is before last_successful_item, and is not new");
+			//di_log(DI_LOG_LEVEL_DEBUG, "menu item (%d) is before last_successful_item (%d), and is not new", p->installer_menu_item, last_successful_item);
 			continue;
 		}
 		/* If menutest says this item should be default, make it so */
@@ -222,23 +222,58 @@ get_default_menu_item(di_slist *list)
 	return NULL;
 }
 
-/* Return the text of the menu entry for PACKAGE.  */
-static size_t menu_entry(struct debconfclient *debconf, di_system_package *package, char *buf, size_t size)
+#define menu_entry_maxlen 256
+
+/* Returns a the text of the menu entry for PACKAGE (in a buffer
+ * that will persist until the next call of the function) */
+const char *menu_entry(struct debconfclient *debconf, di_system_package *package)
 {
-	char question[256];
+	char question[menu_entry_maxlen];
+	static char buf[menu_entry_maxlen];
 
 	snprintf(question, sizeof(question), "debian-installer/%s/title", package->p.package);
 	if (!debconf_metaget(debconf, question, "Description")) {
-		strncpy(buf, debconf->value, size);
-		return strlen (buf);
+		strncpy(buf, debconf->value, menu_entry_maxlen);
+		return buf;
 	}
 
 	/* The following fallback case can go away once all packages
 	   have transitioned to the new form.  */
 	di_log(DI_LOG_LEVEL_INFO, "Falling back to the package description for %s", package->p.package);
 	if (package->p.short_description)
-		strncpy(buf, package->p.short_description, size);
-	return strlen (buf);
+		strncpy(buf, package->p.short_description, menu_entry_maxlen);
+	else
+		buf[0]='\0';
+	return buf;
+}
+
+/* Escapes a string so it can be added to a Choices list. */
+const char *choices_escape(const char *string) {
+	static char buf[menu_entry_maxlen * 2];
+	const char *s;
+	char *p;
+
+	s = strchr(string, ',');
+	if (s == NULL)
+		return string;
+
+	/* This is not the cheapest way to do it, but a comma in a
+	 * choice is very rare. */
+	s=string;
+	p=buf;
+	while (s[0] != '\0') {
+		if (s[0] == ',') {
+			p[0] = '\\';
+			p++;
+		}
+
+		p[0]=s[0];
+
+		p++;
+		s++;
+	}
+	p[0] = '\0';
+	return buf;
 }
 
 /* Priority at which the menu should be displayed */
@@ -269,7 +304,8 @@ di_system_package *show_main_menu(di_packages *packages, di_packages_allocator *
 	di_slist_node *node;
 	di_system_package *menudefault = NULL, *ret = NULL;
 	int i = 0, num = 0;
-	char buf[256], *menu, *s;
+	char *menu, *s;
+	const char *buf;
 	int menu_prio, menu_size, menu_used, size;
 
 	for (node = packages->list.head; node; node = node->next) {
@@ -295,16 +331,17 @@ di_system_package *show_main_menu(di_packages *packages, di_packages_allocator *
 	/*
 	 * Generate list of menu choices for debconf.
 	 */
-	menu = di_malloc(1024);
-	menu[0] = '\0';
 	menu_size = 1024;
+	menu = di_malloc(menu_size);
+	menu[0] = '\0';
 	menu_used = 1;
 	for (node = list->head; node != NULL; node = node->next) {
 		p = node->data;
 		if (!p->installer_menu_item ||
 		    !isinstallable(p))
 			continue;
-		size = menu_entry(debconf, p, buf, sizeof (buf));
+		buf = choices_escape(menu_entry(debconf, p));
+		size = strlen(buf);
 		if (menu_used + size + 2 > menu_size)
 		{
 			menu_size += 1024;
@@ -323,7 +360,7 @@ di_system_package *show_main_menu(di_packages *packages, di_packages_allocator *
 	debconf_capb(debconf);
 	debconf_subst(debconf, MAIN_MENU, "MENU", menu);
 	if (menudefault) {
-		menu_entry(debconf, menudefault, buf, sizeof (buf));
+		buf=menu_entry(debconf, menudefault);
 		debconf_set(debconf, MAIN_MENU, buf);
 	}
 	else {
@@ -339,7 +376,7 @@ di_system_package *show_main_menu(di_packages *packages, di_packages_allocator *
 	/* Figure out which menu item was selected. */
 	for (i = 0; i < num; i++) {
 		p = package_array[i];
-		menu_entry(debconf, p, buf, sizeof (buf));
+		buf = menu_entry(debconf, p);
 		if (strcmp(buf, s) == 0) {
 			ret = p;
 			break;
@@ -370,7 +407,8 @@ di_system_package *show_main_menu(di_packages *packages, di_packages_allocator *
 static int satisfy_virtual(di_system_package *p) {
 	di_slist_node *node;
 	di_system_package *dep, *defpkg = NULL;
-	char buf[256], *menu, *s = NULL;
+	char *menu, *s = NULL;
+	const char *buf;
 	size_t menu_size, menu_used, size;
 	int is_menu_item = 0;
 
@@ -416,7 +454,8 @@ static int satisfy_virtual(di_system_package *p) {
 		if (dep->installer_menu_item)
 			is_menu_item = 1;
 
-		size = menu_entry(debconf, dep, buf, sizeof (buf));
+		buf = choices_escape(menu_entry(debconf, dep));
+		size = strlen(buf);
 		if (menu_used + size + 2 > menu_size)
 		{
 			menu_size += 1024;
@@ -449,7 +488,7 @@ static int satisfy_virtual(di_system_package *p) {
 			char *priority = "medium";
 			/* Only let the user choose if one of them is a menu item */
 			if (defpkg != NULL) {
-				menu_entry(debconf, defpkg, buf, sizeof(buf));
+				buf = menu_entry(debconf, defpkg);
 				debconf_set(debconf, MISSING_PROVIDE, buf);
 			} else {
 				/* TODO: How to figure out a default? */
@@ -470,7 +509,7 @@ static int satisfy_virtual(di_system_package *p) {
 		for (node = p->p.depends.head; node; node = node->next) {
 			di_package_dependency *d = node->data;
 			dep = (di_system_package *)d->ptr;
-			menu_entry(debconf, dep, buf, sizeof(buf));
+			buf = menu_entry(debconf, dep);
 			if (!is_menu_item || strcmp(s, buf) == 0) {
 				/* Ick. If we have a menu item it has to match the
 				 * debconf choice, otherwise we configure all of
@@ -574,11 +613,11 @@ static void restore_default_priority (void) {
 }
 
 void notify_user_of_failure (di_system_package *p) {
-	char buf[256];
+	const char *buf;
 	
 	set_package_title(p);
 	debconf_capb(debconf);
-	menu_entry(debconf, p, buf, sizeof (buf));
+	buf = menu_entry(debconf, p);
 	debconf_subst(debconf, ITEM_FAILURE, "ITEM", buf);
 	debconf_input(debconf, "critical", ITEM_FAILURE);
 	debconf_go(debconf);
@@ -720,7 +759,7 @@ int main (int argc __attribute__ ((unused)), char **argv) {
 		packages = di_system_packages_status_read_file(DI_SYSTEM_DPKG_STATUSFILE, allocator);
 
 		/* tell cdebconf to save the database */
-		debconf->command(debconf, "X_SAVE", NULL);
+		debconf_x_save(debconf);
 	}
 	
 	return exit_loop != 0 ? EXIT_OK : EXIT_FAILURE;
